/*
 * file Workspace.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM
 *
 * (c) Copyright IBM Corporation 2004, 2008.  All Rights Reserved. 
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or  
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp. 
 */
package javax.wvcm;

import javax.wvcm.PropertyNameList.PropertyName;
import javax.wvcm.ResourceList.ResponseIterator;
import javax.wvcm.WvcmException.ReasonCode;

/**
 * A proxy for a workspace resource.
 *
 * A workspace resource provides an environment within which a user can make persistent modifications
 * to a configuration of resources without interfering or disrupting the work of other users of those resources.
 * This requires that the user be able to persist changes to resources (both content changes and naming changes)
 * without those changes being immediately visible to other users.  To provide this functionality,
 * the persistent changes saved in one workspace (also known as a "sandbox" or a "view") are visible
 * in other workspaces only after the user of that workspace explicitly exposes those changes for use by other workspaces,
 * and after a user of another workspace explicitly requests to see the changes that have been exposed. </p>
 * 
 * @since 1.0
 */
public interface Workspace extends ControllableFolder {

    /**
     * Create a persistent workspace, where the provider can allocate the location
     * for the new workspace.
     * The provider should use the client-specified location if it is valid,
     * but can select a different location if the location is not valid
     * or already identifies a workspace.
     * <p>
     * Postconditions:
     * <li>(initialize-resource): A new persistent workspace resource exists at the location
     *  of this Workspace.
     * @see ControllableResource#doCreateResource
     * 
     * @param feedback Specifies optional feedback to the caller.
     * @return a proxy for the newly created workspace, whose properties are specified by feedback.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#METHOD_NOT_SUPPORTED}:
     *  If the provider does not support the creation of workspaces, this request MUST fail.
     *  A client can determine a valid location for this method
     *  with a {@link Provider#rootLocation()} request.
     */
    public Workspace doCreateGeneratedResource(Feedback feedback) throws WvcmException;

    /**
     * This property identifies a folder in the workspace that can contain bound members
     * that are not folders.
     * <p>
     * Some workspaces do not allow the creation of non-folder resources
     * as bound members of the workspace itself, and instead reserve bound members
     * of the workspace for folders that represent different repositories or configuration roots.
     */
    public static final PropertyName<ControllableFolder> CONTROLLABLE_RESOURCE_HOME =
        new PropertyName<ControllableFolder>("controllable-resource-home"); //$NON-NLS-1$

    /**
     * Get the {@link #CONTROLLABLE_RESOURCE_HOME} property.
     * 
     * @return the {@link #CONTROLLABLE_RESOURCE_HOME} property.
     * @throws WvcmException if this Workspace was not created with
     * {@link #CONTROLLABLE_RESOURCE_HOME} as a wanted property.
     */
    public ControllableFolder getControllableResourceHome() throws WvcmException;

    /**
     * This property identifies a folder in the workspace that can contain bound members
     * that are baseline-controlled folders.
     * This property returns <code>null</code> if this Workspace does not support
     * baseline-controlled folders.
     */
    public static final PropertyName<ControllableFolder> CONFIGURATION_ROOT_FOLDER_HOME =
        new PropertyName<ControllableFolder>("configuration-root-folder-home"); //$NON-NLS-1$

    /**
     * Get the {@link #CONFIGURATION_ROOT_FOLDER_HOME} property.
     * 
     * @return the {@link #CONFIGURATION_ROOT_FOLDER_HOME} property.
     * @throws WvcmException if this Workspace was not created with
     * {@link #CONFIGURATION_ROOT_FOLDER_HOME} as a wanted property.
     */
    public ControllableFolder getConfigurationRootFolderHome() throws WvcmException;

    /**
     * This property identifies a list of folders that can contain activities for this workspace.
     * This list is empty if this Workspace does not provide activity support.
     */
    public static final PropertyName<ResourceList<Folder>> ACTIVITY_FOLDER_LIST =
        new PropertyName<ResourceList<Folder>>("activity-folder-list"); //$NON-NLS-1$

    /**
     * Get the {@link #ACTIVITY_FOLDER_LIST} property.
     * 
     * @return the {@link #ACTIVITY_FOLDER_LIST} property.
     * @throws WvcmException if this Workspace was not created with
     * {@link #ACTIVITY_FOLDER_LIST} as a wanted property.
     */
    public ResourceList<Folder> getActivityFolderList() throws WvcmException;

    /**
     * This property identifies a list of folders that contain the version
     * history resources for the version-controlled resources in this workspace.
     */
    public static final PropertyName<ResourceList<Folder>> VERSION_HISTORY_FOLDER_LIST =
        new PropertyName<ResourceList<Folder>>("version-history-folder-list"); //$NON-NLS-1$

    /**
     * Get the {@link #VERSION_HISTORY_FOLDER_LIST} property.
     * 
     * @return the {@link #VERSION_HISTORY_FOLDER_LIST} property.
     * @throws WvcmException if this Workspace was not created with
     * {@link #VERSION_HISTORY_FOLDER_LIST} as a wanted property.
     */
    public ResourceList<Folder> getVersionHistoryFolderList() throws WvcmException;

    /**
     * The list of root folders of configurations in this workspace.
     */
    public static final PropertyName<ResourceList<ControllableFolder>> BASELINE_CONTROLLED_FOLDER_LIST =
        new PropertyName<ResourceList<ControllableFolder>>("baseline-controlled-folder-list"); //$NON-NLS-1$

    /**
     * Get the {@link #BASELINE_CONTROLLED_FOLDER_LIST} property.
     * 
     * @return the {@link #BASELINE_CONTROLLED_FOLDER_LIST} property.
     * @throws WvcmException if this Workspace was not created with
     * {@link #BASELINE_CONTROLLED_FOLDER_LIST} as a wanted property.
     */
    public ResourceList<ControllableFolder> getBaselineControlledFolderList()
    throws WvcmException;

    /**
     * The activity that currently is being performed in this workspace.
     * <p>
     * When a member of this workspace is
     * checked out, if no activity is specified in the checkout request, the
     * {@link #CURRENT_ACTIVITY} will be used.  This allows an activity-unaware
     * client to update a workspace in which activity tracking is required. </p>
     * <p>
     * The {@link #CURRENT_ACTIVITY} of a controlled workspace can be modified
     * while the workspace is checked in.
     * @see #getCurrentActivity
     * @see #setCurrentActivity
     */
    public static final PropertyName<Activity> CURRENT_ACTIVITY =
        new PropertyName<Activity>("current-activity"); //$NON-NLS-1$

    /**
     * Get the {@link #CURRENT_ACTIVITY} property.
     * 
     * @return the {@link #CURRENT_ACTIVITY} property.
     * @throws WvcmException if this property was not set and
     * this Workspace was not created with
     * {@link #CURRENT_ACTIVITY} as a wanted property.
     * @see #setCurrentActivity
     */
    public Activity getCurrentActivity() throws WvcmException;

    /**
     * Set the {@link #CURRENT_ACTIVITY} property.
     * 
     * @param activity the {@link Activity} object that
     * identifies the new {@link #CURRENT_ACTIVITY} for this Workspace.
     * @see #getCurrentActivity
     */
    public void setCurrentActivity(Activity activity);

    /**
     * The stream of this workspace.
     * <p>
     * When a controllable resource in this workspace is checked in,
     * the new version that is created is automatically added to the {@link Stream#VERSION_LIST}
     * of the {@link #STREAM} of the workspace.
     * When a configuration of this workspace is
     * checked out, if no stream is specified in the checkout request, the
     * {@link #STREAM} will be used. </p>
     * <p>
     * An implementation may refuse to modify the {@link #STREAM} of an existing workspace.
     * @see #getStream
     * @see #setTarget
     * @see #setIsolatedTarget
     */
    public static final PropertyName<Stream> STREAM =
        new PropertyName<Stream>("stream"); //$NON-NLS-1$

    /**
     * Get the {@link #STREAM} property.
     * 
     * @return the {@link #STREAM} property.
     * @throws WvcmException if this property was not set and
     * this Workspace was not created with
     * {@link #STREAM} as a wanted property.
     * @see #setTarget
     * @see #setIsolatedTarget
     */
    public Stream getStream() throws WvcmException;

    /**
     * The current target stream of this workspace.
     * <p>
     * It is expected that the target stream periodically
     * will be merged into and updated by this workspace.
     * The target stream of a workspace MAY be the {@link #STREAM} of that workspace.
     * and therefore checking in a controllable resource in the workspace MAY
     * (but is not guaranteed to) change the {@link Stream#VERSION_LIST} of the target stream.
     * <p>
     * An implementation may refuse to modify the {@link #TARGET} of an existing workspace.
     * @see #getTarget
     * @see #setTarget
     */
    public static final PropertyName<Stream> TARGET =
        new PropertyName<Stream>("target"); //$NON-NLS-1$

    /**
     * Get the {@link #TARGET} property.
     * 
     * @return the {@link #TARGET} property.
     * @throws WvcmException if this property was not set and
     * this Workspace was not created with {@link #TARGET} as a wanted property.
     * @see #setTarget
     */
    public Stream getTarget() throws WvcmException;

    /**
     * Set the {@link #TARGET} property.
     * 
     * @param stream the {@link Stream} object that
     * identifies the {@link #TARGET} for this Workspace.
     * @see #getTarget
     */
    public void setTarget(Stream stream);

    /**
     * The current isolated target stream of this workspace.
     * <p>
     * It is expected that the isolated target stream periodically
     * will be merged into and updated by this workspace.
     * The isolated target stream of a workspace MUST NOT be the {@link #STREAM} of that workspace,
     * and therefore checking in a controllable resource in the workspace is guaranteed
     * not to change the {@link Stream#VERSION_LIST} of the isolated target stream.
     * <p>
     * An implementation may refuse to modify the {@link #ISOLATED_TARGET} of an existing workspace.
     * @see #getIsolatedTarget
     * @see #setIsolatedTarget
     */
    public static final PropertyName<Stream> ISOLATED_TARGET =
        new PropertyName<Stream>("isolated-target"); //$NON-NLS-1$

    /**
     * Get the {@link #ISOLATED_TARGET} property.
     * 
     * @return the {@link #ISOLATED_TARGET} property.
     * @throws WvcmException if this property was not set and
     * this Workspace was not created with {@link #ISOLATED_TARGET} as a wanted property.
     * @see #setIsolatedTarget
     */
    public Stream getIsolatedTarget() throws WvcmException;

    /**
     * Set the {@link #ISOLATED_TARGET} property.
     * 
     * @param stream the {@link Stream} object that
     * identifies the {@link #ISOLATED_TARGET} for this Workspace.
     * @see #getIsolatedTarget
     */
    public void setIsolatedTarget(Stream stream);

    /**
     * The list of current source streams of this workspace.
     * <p>
     * It is expected that a source stream periodically will be merged into this workspace.
     * <p>
     * An implementation may refuse to modify the {@link #SOURCE_LIST} of an existing workspace.
     * @see #getSourceList
     * @see #setSourceList
     */
    public static final PropertyName<ResourceList<Stream>> SOURCE_LIST =
        new PropertyName<ResourceList<Stream>>("source-list"); //$NON-NLS-1$

    /**
     * Get the {@link #SOURCE_LIST} property.
     * 
     * @return the {@link #SOURCE_LIST} property.
     * @throws WvcmException if this property was not set and
     * this Workspace was not created with {@link #SOURCE_LIST} as a wanted property.
     * @see #setSourceList
     */
    public ResourceList<Stream> getSourceList() throws WvcmException;

    /**
     * Set the {@link #SOURCE_LIST} property.
     * 
     * @param streamList the list of {@link Stream} objects that
     * identifies the {@link #SOURCE_LIST} for this Workspace.
     * @see #getSourceList
     */
    public void setSourceList(ResourceList<Stream> streamList);

    /** 
     * This property identifies each checked-out ControlledResource 
     * in this workspace. 
     */
    public static final PropertyName<ResourceList<ControllableResource>> WORKSPACE_CHECKOUT_LIST =
        new PropertyName<ResourceList<ControllableResource>>("workspace-checkout-list"); //$NON-NLS-1$

    /**
     * Get the {@link #WORKSPACE_CHECKOUT_LIST} property.
     * 
     * @return the {@link #WORKSPACE_CHECKOUT_LIST} property.
     * @throws WvcmException if this Workspace was not created with
     * {@link #WORKSPACE_CHECKOUT_LIST} as a wanted property.
     */
    public ResourceList<ControllableResource> getWorkspaceCheckoutList() throws WvcmException;

    /**
     * Apply {@link ControllableResource#doVersionControl}
     * to each element of the resourceList.
     * 
     * @param resourceList the list of resources to be put under version control.
     * @param feedback Specifies optional feedback to the caller.
     * @return A new proxy for this resource, whose properties are specified by feedback.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#MULTI_STATUS}:
     *  At least one resource in the list could not be put under version control.
     *  If any of the resources in the resourceList cannot be put under version control,
     *  the provider MAY fail the request.
     */
    public <T extends Resource> Workspace doVersionControl(
            ResourceList<T> resourceList,
            Feedback feedback) throws WvcmException;

    /**
     * Apply {@link ControllableResource#doCheckin} to each element of the resourceList.
     * 
     * @param resourceList the list of resources to be checked in.
     * @param comment A comment for any new version whose controllable resource has no comment.
     * @param flags passed through to {@link ControllableResource#doCheckin doCheckin}.
     * @param feedback Specifies optional feedback to the caller.
     * @return A new proxy for this resource, whose properties are specified by feedback.
     * @throws WvcmException ReasonCode:
     * <li>See {@link ControllableResource#doCheckin}
     * <li>{@link ReasonCode#MULTI_STATUS}:
     *  At least one resource in the list could not be checked in.
     *  If any of the checked-out resources in the resourceList cannot be checked in,
     *  the provider MAY fail the request.
     */
    public <T extends Resource> Workspace doCheckin(
            ResourceList<T> resourceList,
            String comment,
            CheckinFlag[] flags, Feedback feedback) throws WvcmException;

    /**
     * Apply {@link ControllableResource#doCheckin} to each resource in {@link #WORKSPACE_CHECKOUT_LIST}.
     * @param comment A comment for any new version whose controllable resource has no comment.
     * @param flags passed through to {@link ControllableResource#doCheckin doCheckin}.
     * @param feedback Specifies optional feedback to the caller.
     * @return A new proxy for this resource, whose properties are specified by feedback.
     * @throws WvcmException ReasonCode:
     * <li>See {@link ControllableResource#doVersionControl}
     * <li>{@link ReasonCode#MULTI_STATUS}:
     *  At least one of the checked out resources could not be checked in.
     *  If any of the checked-out resources in the {@link Workspace#WORKSPACE_CHECKOUT_LIST} cannot be checked in,
     *  the provider MAY fail the request.
     */
    public Workspace doCheckinAll(
            String comment,
            CheckinFlag[] flags, Feedback feedback) throws WvcmException;

    /**
     * A description of how the merge would affect the merge target when the merge source
     * is neither an ancestor nor a descendant of the {@link ControllableResource#CHECKED_IN}
     * or {@link ControllableResource#CHECKED_OUT} version of the merge target.
     */
    public interface MergePreviewReport {
        /**
         * Get the merge target resource.
         * 
         * @return a version-controlled resource that would be
         * updated by the merge request.
         */
        public ControllableResource getTarget();

        /**
         * Get the common ancestor for the merges.
         * 
         * @return a version that is the nearest common ancestor of all
         * the versions to be merged.
         */
        public Version getCommonAncestor();

        /**
         * Get the list of versions whose controlled state is to be merged with the target.
         * 
         * @return a list of {@link Version} objects that identifies the versions
         * to be merged to the specified target.
         */
        public ResourceList<Version> getVersionList();
    }


    /**
     * Preview a merge.
     * Because automerging of folders can affect what resources require a merge,
     * the result from this request may not correspond exactly to the result of an actual merge.
     * 
     * @param sourceList a list of {@link Resource} objects that identify
     * the versions to be merged.  See the sourceList argument for {@link #doMerge}.
     * @param feedback the properties available in the returned proxies.
     * @return an iterator of MergePreviewReport objects,
     * indicating resources that have their {@link ControllableResource#MERGE_LIST}
     * or {@link ControllableResource#AUTO_MERGE_LIST} set by a {@link #doMerge} request
     * with {@link MergeFlag#NO_AUTO_MERGE} specified.
     * @throws WvcmException ReasonCode:
     *  <li>{@link ReasonCode#METHOD_NOT_SUPPORTED}:
     *  This resource does not support this report.
     */
    public <T extends Resource> ResponseIterator<MergePreviewReport> doMergePreviewReport(
            ResourceList<T> sourceList,
            Feedback feedback)
    throws WvcmException;

    /** Boolean flags for the doMerge method */
    public static enum MergeFlag
    {
        /** 
         * Indicates whether to fork on checkout even if forking is discouraged.
         * @see ControllableResource.CheckoutFlag#FORK_OK
         */
        FORK_OK("fork-ok"), //$NON-NLS-1$

        /**
         * Indicates whether to do a checkout even
         * if there already is a reserved checkout from the currently selected
         * version in the checkout activity.
         * @see ControllableResource.CheckoutFlag#RESERVED
         */
        @SuppressWarnings("hiding") //$NON-NLS-1$
        RESERVED("reserved"), //$NON-NLS-1$

        /**
         * Indicates whether the provider is allowed is allowed to perform automatic merges.
         */
        NO_AUTO_MERGE("no-auto-merge"), //$NON-NLS-1$

        /**
         * Indicates whether the provider is allowed to checkout files for merging.
         */
        NO_CHECKOUT("no-checkout"), //$NON-NLS-1$

        /**
         * Indicates whether a checkin will be performed if the persistent state
         * of the new version would be identical to that of the previous
         * version. If this flag is not set, a checkin operation on an unchanged
         * controllable resource will uncheckout that resource.
         */
        CHECKIN_IDENTICAL("checkin-identical"), //$NON-NLS-1$

        /**
         * Indicates whether a stream source should be updated with the current
         * state of the workspace if the merge succeeds.
         */
        UPDATE_STREAM("update-stream"); //$NON-NLS-1$

        private MergeFlag(String flagName) { _flagName = flagName; }

        /**
         * Returns a string representation of this flag suitable for diagnostics.
         */
        @Override
        public String toString() { return _flagName; }

        /**
         * The string form of this flag.
         */
        private final String _flagName;
    }


    /**
     * Merge the versions specified in the <code>sourceList</code> 
     * into the appropriate target in this {@link Workspace}.
     * The "target" of a particular source version is the version-controlled 
     * member of the workspace identified by this Workspace
     * that is associated with the version history for that version.
     * <br>If the target is checked-in, and the source version is a descendant
     * of the target version, the target is updated to the source version.
     * <br>If the target is checked-in, and the source version is an ancestor
     * of the target version, the target is left unmodified.
     * <br>If the target is checked-out, or if the target version is on a different
     * line of descent than the source version, the target is checked out
     * (if it is not already checked-out), and the source version is either
     * added to the {@link #MERGE_LIST} or the {@link #AUTO_MERGE_LIST} of the target.
     * If a source version is added to the {@link #AUTO_MERGE_LIST}, the provider must
     * have updated the content and controlled properties of the target to contain the results of merging
     * the content and controlled properties of the source version with the content
     * and controlled properties of the target at the time of the merge.
     * <p>
     * Postconditions:
     * <li>(merge-sources): Each of the versions in the sourceList MUST have been
     *  merged into their target in this Workspace.  If a source has no target 
     *  in this Workspace, it is ignored.
     * <li>(ancestor-version): If the merge target is a version-controlled resource
     *  whose {@link #CHECKED_IN} version or {@link #CHECKED_OUT} version is the merge source
     *  or is a descendant of the merge source, the merge target MUST NOT have been modified by the doMerge.
     * <li>(descendant-version): If the merge target was a checked-in version-controlled resource
     *  whose {@link #CHECKED_IN} version was an ancestor of the merge source,
     *  a doUpdate operation MUST have been applied to the merge target.
     * <li>(checked-out-for-merge): If the merge target was a checked-in version-controlled resource
     *  whose {@link #CHECKED_IN} version was neither a descendant nor an ancestor of the merge source,
     *  a doCheckout MUST have been applied to the merge target.
     *  All arguments that could appear in a doCheckout request MUST have been used
     *  as arguments to the doCheckout request.  
     * <li>(update-merge-list): If the {@link #CHECKED_OUT} version of the merge target is not equal to
     *  or a descendant of the merge source, the merge source MUST be added to either
     *  the {@link #MERGE_LIST} or the {@link #AUTO_MERGE_LIST} of the merge target.
     *  The merge target MUST appear in the result.  If a merge source has been added
     *  to the {@link #AUTO_MERGE_LIST}, the controlled state of the merge target MUST have been modified
     *  by the provider to reflect the result of a logical merge of the merge source and the merge target.
     *  If a merge source has been added to the {@link #MERGE_LIST}, the controlled state of the merge target
     *  MUST NOT have been modified by the provider.  If {@link MergeFlag#NO_AUTO_MERGE} is specified in the request,
     *  the merge source MUST NOT have been added to the {@link #AUTO_MERGE_LIST}.
     * <li>(merge-baseline): If {@link MergeFlag#NO_AUTO_MERGE} is specified and the merge target is a configuration
     *  whose {@link #CHECKED_OUT} baseline is neither a descendant or an ancestor
     *  of the source baseline, then the merge baseline MUST have been
     *  added to the {@link #MERGE_LIST} of the configuration.
     * <li>(auto-merge-baseline): If {@link MergeFlag#NO_AUTO_MERGE} is not specified and the merge target is a configuration
     *  whose {@link #CHECKED_OUT} baseline is neither a descendant or an ancestor
     *  of the source baseline, then the merge baseline MUST have been
     *  added to the {@link #AUTO_MERGE_LIST} of the configuration, and
     *  each version of that baseline MUST have been merged into the workspace 
     *  of that configuration.
     * <li>(update-version-controlled-folder-members): Same semantics as {@link #doUpdate}.
     * 
     * @param sourceList a list of {@link Resource} objects that
     * identifies the versions to be merged.
     * If a member of sourceList is an activity but not a stream,
     * each version in the {@link Activity#ACTIVITY_VERSION_LIST} is a source version.
     * If a member of sourceList is a stream, each version in the {@link Stream#VERSION_LIST}
     * is a source version.
     * If a member of sourceList is a version-controlled resource,
     * the checked-in version of the member is a source version;
     * if that version-controlled resource is a folder, all members
     * of that folder are considered members of the sourceList.
     * @param flags Boolean flags for doMerge.
     * @param feedback Specifies optional feedback to the caller.
     * @return a new proxy for this resource, whose properties are specified by feedback.
     * @throws WvcmException An implementation may throw a single exception on the first error encountered,
     * or may attempt to merge all of the source versions, with any errors accumulated in the
     * {@link WvcmException#getNestedExceptions} list.
     * <br>ReasonCode:
     * <li>{@link ReasonCode#BAD_ARGUMENT_TYPE}:
     * The resource identified by the <code>activity</code> argument must be an activity.
     * <li>{@link ReasonCode#MUST_BE_CHECKED_IN}:
     *  A version-controlled resource in the sourceList must be checked in.
     * <li>{@link ReasonCode#CANNOT_MERGE_CHECKOUT_NOT_ALLOWED}:
     *  If {@link MergeFlag#NO_CHECKOUT} is specified in the request,
     *  it MUST be possible to perform the merge without checking out the merge target.
     *  <li>All reason codes of {@link ControllableResource#doCheckout} apply to any checkout performed by the request.
     *  
     */
    public <T extends Resource> Workspace doMerge(
            ResourceList<T> sourceList,
            MergeFlag[] flags,
            Feedback feedback)
    throws WvcmException;

    /**
     * Update the state of the appropriate target in this {@link Workspace}.
     * The "target" of a particular source version is the version-controlled 
     * member of the workspace identified by this Workspace
     * that is associated with the version history for that version.
     * <p>
     * Postconditions:
     * <li>(update-content-and-properties): The content and controlled properties of a target
     *  MUST be the same as that of the version specified by the source version.
     * <li>(update-version-controlled-folder-members): If the request modified the {@link #CHECKED_IN} version
     *  of a version-controlled folder, then the version-controlled members
     *  of that version-controlled folder MUST have been updated.  In particular:
     *  <br>- A version-controlled bound member MUST have been deleted if its version history
     *    is not identified by the {@link Folder#CHILD_LIST} of the new {@link #CHECKED_IN} version.
     *  <br>- A version-controlled bound member for a given version history
     *    MUST have been renamed if its binding name differs from the Binding name
     *    for that version history in the {@link Folder#CHILD_LIST} of the new {@link #CHECKED_IN} version.
     *  <br>- A new version-controlled bound member MUST have been created when a version history
     *    is identified by the {@link Folder#CHILD_LIST} of the {@link #CHECKED_IN} version,
     *    but there was no member of the version-controlled folder for that version history.
     *    If a new version-controlled member is in a workspace that already has
     *    a version-controlled resource for that version history, then the new version-controlled member
     *    MUST be just a binding (i.e. another name for) that existing version-controlled resource.
     *    Otherwise, the content and controlled properties of the new version-controlled member MUST have been initialized
     *    to be those of the version specified for that version history by the request.
     *    If no version is specified for that version history by the request, the version selected is provider defined.
     * <li>(set-baseline-controlled-folder-members): If the request updated the {@link #CHECKED_IN} property
     *  of a configuration to identify a new baseline, then each version-controlled member of the {@link Configuration#ROOT_FOLDER}
     *  of that configuration MUST have been updated so its {@link #CHECKED_IN} property identifies the
     *  corresponding version in the {@link Baseline#VERSION_LIST} of that baseline.  In particular:
     * <br>- A version-controlled member for a given version history MUST have been deleted
     *  if there is no version for that version history in the {@link Baseline#VERSION_LIST} of the baseline.
     * <br>- A version-controlled member for a given version history MUST have been renamed if its name
     *  relative to the baseline-controlled folder is different from that of the version-controlled member
     *  for that version history when the baseline was created.
     * <br>- A new version-controlled member MUST have been created for each member of the {@link Baseline#VERSION_LIST} 
     *  of the baseline for which there is no corresponding version-controlled member
     *  in the baseline-controlled folder.  The name of that new member relative to the baseline-controlled folder is
     *  the name of the version-controlled member for that version history when the baseline was created.
     * <br>- A doUpdate request MUST have been applied to each version-controlled member
     *  for a given version history whose {@link #CHECKED_IN} version is not the version
     *  identified in the {@link Baseline#VERSION_LIST} of the baseline.
     * <li> (update-streams): If the {@link MergeFlag#UPDATE_STREAM} is specified,
     *  and if the merge succeeds, each stream in the sourceList is updated with
     *  the current configuration of this workspace.
     *  
     * @param sourceList the list of resources specifying the versions whose target in this 
     * workspace should be updated.  See the sourceList argument for {@link #doMerge}.
     * @param feedback Specifies optional feedback to the caller.
     * @return a new proxy for this resource, whose properties are specified by feedback.
     * @throws WvcmException ReasonCode:
     * <li>{@link ReasonCode#MUST_BE_CHECKED_IN}:
     *  A version-controlled resource in the sourceList must be checked in.
     * <li>{@link ReasonCode#NO_CHECKED_OUT_BASELINE_CONTROLLED_FOLDER_MEMBERS}:
     *  If the target identifies a configuration, then all version-controlled members
     *  of the {@link Configuration#ROOT_FOLDER} of that configuration MUST be checked-in.
     */
    public <T extends Resource> Workspace doUpdate(ResourceList<T> sourceList, Feedback feedback)
    throws WvcmException;

}
